from unittest import TestCase
from assetsstore.assets import FileAssets
import glob
import os
import logging


class AsssetsLocalTest(TestCase):
    def setUp(self):
        self.maxDiff = None
        logging.basicConfig(level=logging.INFO)

    def test_no_asset_store_set(self):
        os.environ["ASSET_STORE"] = ""
        with self.assertRaises(Exception) as context:
            FileAssets.get_asset()
        print(context.exception)
        self.assertTrue("Invalid ASSET_STORE value" in str(context.exception))

    def test_upload_and_download_from_local(self):
        # get set store
        os.environ["ASSET_STORE"] = "LocalFiles"

        os.environ["ASSET_ACCESS_KEY_ID"] = ""
        os.environ["ASSET_SECRET_ACCESS_KEY"] = ""
        os.environ["ASSET_LOCATION"] = "assetsstore/tests/results/remote/"
        os.environ["ASSET_REGION"] = ""

        os.environ["LOCAL_STORE"] = "assetsstore/tests/fixtures/"
        handler = FileAssets.get_asset()
        self.assertEqual(True, handler.put_file("test.txt"))

        os.environ["LOCAL_STORE"] = "assetsstore/tests/results/"
        handler = FileAssets.get_asset()
        self.assertEqual(True, handler.get_file("test.txt"))

        # get again to check if it exists
        self.assertEqual(True, handler.get_file("test.txt"))

        # delete remote file
        self.assertEqual(True, handler.del_file("test.txt"))

        # delete local copy
        self.assertEqual(True, handler.del_local_file("test.txt"))

    def tearDown(self):
        for file in glob.glob("results/*"):
            if ".gitkeep" not in file:
                os.remove(file)
        for file in glob.glob("results/remote/*"):
            if ".gitkeep" not in file:
                os.remove(file)



from assetsstore.assets import FileAssets
from shutil import copyfile
import pathlib
import logging
import os

logger = logging.getLogger(__name__)


class LocalFiles(FileAssets):
    """
    A class that represents local file assets.

    This class provides methods to interact with local files,
    including getting access to files,
    uploading files, downloading files, and deleting files.
    """

    def __init__(
        self, location=os.getenv("ASSET_LOCATION"), server_url=os.getenv("SERVER_URL")
    ):
        self.location = location
        self.server_url = server_url
        super().__init__()

    def check_if_exists(self, path: str):
        pass

    def get_access(self, filename: str, *args, **kwargs):
        """
        Get the access URL for a file.

        Args:
            filename (str): The name of the file.

        Returns:
            str: The access URL for the file.

        """
        return "{}{}".format(self.server_url, filename)

    def get_upload_access(self, filename: str, *args, **kwargs):
        """
        Get the upload access URL for a file.

        Args:
            filename (str): The name of the file.

        Returns:
            str: The upload access URL for the file.

        """
        return "{}{}".format(self.server_url, filename)

    def get_folder(self, path: str):
        """
        Get the files in a folder.

        Args:
            path (str): The path of the folder.

        Returns:
            bool: True if the operation is successful, False otherwise.

        """
        for root, dirs, files in os.walk("{}{}".format(self.location, path)):
            for f in files:
                self.get_file("{}/{}".format(root.replace(self.location, ""), f))
        return True

    def get_file(self, filename):
        """
        Get a file from the asset store.

        Args:
            filename (str): The name of the file.

        Returns:
            bool: True if the file is successfully downloaded, False otherwise.

        """
        asset_filename = os.path.realpath("{}{}".format(self.location, filename))
        local_filename = os.path.realpath("{}{}".format(self.local_store, filename))
        try:
            local_file = pathlib.Path(local_filename)
            if not local_file.is_file():
                folder_path = pathlib.Path("/".join(local_filename.split("/")[:-1]))
                folder_path.mkdir(parents=True, exist_ok=True)
                copyfile(asset_filename, local_filename)
            else:
                logger.info("File already downloaded {}".format(local_filename))
                return True
        except Exception as e:
            logger.exception(
                "Download file from local store failed with error: {}".format(str(e))
            )
            return False
        return True

    def put_file(self, filename):
        """
        Put a file into the asset store.

        Args:
            filename (str): The name of the file.

        Returns:
            bool: True if the file is successfully uploaded, False otherwise.

        """
        asset_filename = os.path.realpath("{}{}".format(self.location, filename))
        local_filename = os.path.realpath("{}{}".format(self.local_store, filename))
        try:
            folder_path = pathlib.Path("/".join(asset_filename.split("/")[:-1]))
            folder_path.mkdir(parents=True, exist_ok=True)
            copyfile(local_filename, asset_filename)
        except Exception as e:
            logger.exception(
                "Upload file to store failed with error: {}".format(str(e))
            )
            return False
        return True

    def del_file(self, filename):
        """
        Delete a file from the asset store.

        Args:
            filename (str): The name of the file.

        Returns:
            bool: True if the file is successfully deleted, False otherwise.

        """
        asset_filename = os.path.realpath("{}{}".format(self.location, filename))
        if os.path.exists(asset_filename):
            try:
                os.remove(asset_filename)
                return True
            except Exception as e:
                logger.exception(
                    "Delete file from local store failed with error: {}".format(str(e))
                )
        return False



from assetsstore.assets import FileAssets
import pathlib
import logging
import paramiko
import os
import stat


logger = logging.getLogger(__name__)


class ServerFiles(FileAssets):
    """
    Represents a class for managing server files.

    This class provides methods for accessing, uploading, downloading,
    and deleting files on a remote server.

    Attributes:
        server (str): The server address.
        username (str): The username for authentication.
        location (str): The location of the asset files on the server.
        server_url (str): The public URL of the asset server.
        ssh (paramiko.SSHClient): The SSH client for connecting to the server.
        pkey (paramiko.Ed25519Key): The private key for authentication (optional).
        password (str): The password for authentication (optional).
    """

    def __init__(
        self,
        server=os.getenv("ASSET_SERVER"),
        username=os.getenv("ASSET_ACCESS_KEY"),
        location=os.getenv("ASSET_LOCATION"),
        server_url=os.getenv("ASSET_PUBLIC_URL"),
    ):
        self.server = server
        self.username = username
        self.location = location
        self.server_url = server_url
        self.ssh = paramiko.SSHClient()
        self.ssh.load_host_keys(
            os.path.expanduser(os.path.join("~", ".ssh", "known_hosts"))
        )
        pkey_path = os.getenv("ASSET_PRIVATE_KEY")
        if pkey_path:
            self.pkey = paramiko.Ed25519Key.from_private_key_file(pkey_path)
            self.ssh.connect(
                self.server, username=self.username, pkey=self.pkey, banner_timeout=200
            )
        else:
            self.password = os.getenv("ASSET_SECRET_ACCESS_KEY")
            self.ssh.connect(
                self.server, username=self.username, password=self.password
            )
        super().__init__()

    def check_if_exists(self, path: str):
        pass

    def get_access(self, filename, *args, **kwargs):
        """
        Get the access URL for a file.

        Args:
            filename (str): The name of the file.

        Returns:
            str: The access URL for the file.
        """
        return "{}{}".format(self.server_url, filename)

    def get_upload_access(self, filename, *args, **kwargs):
        """
        Get the upload access URL for a file.

        Args:
            filename (str): The name of the file.

        Returns:
            str: The upload access URL for the file.
        """
        return "{}{}".format(self.server_url, filename)

    def get_size(self, folder):
        """
        Get the size of a folder on the server.

        Args:
            folder (str): The path to the folder.

        Returns:
            int: The size of the folder in bytes.
        """
        size = 0
        asset_folder = "{}{}".format(self.location, folder)
        sftp = self.ssh.open_sftp()
        for i in sftp.listdir_attr(asset_folder):
            size = i.st_size
        return size

    def listdir_r(self, sftp, remotedir):
        """
        Recursively list all files in a directory on the server.

        Args:
            sftp (paramiko.SFTPClient): The SFTP client.
            remotedir (str): The path to the directory.

        Returns:
            list: A list of file paths.
        """
        file_list = []
        for entry in sftp.listdir_attr(remotedir):
            remotepath = remotedir + "/" + entry.filename
            mode = entry.st_mode
            if stat.S_ISDIR(mode):
                file_list.extend(self.listdir_r(sftp, remotepath))
            elif stat.S_ISREG(mode):
                file_list.append(remotepath)
        return file_list

    def get_folder(self, path):
        """
        Download a folder from the server.

        Args:
            path (str): The path to the folder on the server.

        Returns:
            bool: True if the folder was downloaded successfully, False otherwise.
        """
        try:
            sftp = self.ssh.open_sftp()
            asset_folder = "{}{}".format(self.location, path)
            for r_file in self.listdir_r(sftp, asset_folder):
                folder_path = pathlib.Path("/".join(r_file.split("/")[:-1]))
                folder_path.mkdir(parents=True, exist_ok=True)
                self.get_file(r_file.replace(self.location, ""))
            sftp.close()
        except Exception as e:
            logger.exception(
                "Failed to read remote folder. Exception {}".format(str(e))
            )
            return False
        return True

    def get_file(self, filename):
        """
        Download a file from the server.

        Args:
            filename (str): The name of the file on the server.

        Returns:
            bool: True if the file was downloaded successfully, False otherwise.
        """
        asset_filename = "{}{}".format(self.location, filename)
        local_filename = os.path.realpath("{}{}".format(self.local_store, filename))
        try:
            local_file = pathlib.Path(local_filename)
            if not local_file.is_file():
                sftp = self.ssh.open_sftp()
                folder_path = pathlib.Path(
                    "{}{}".format(self.local_store, "/".join(filename.split("/")[:-1]))
                )
                folder_path.mkdir(parents=True, exist_ok=True)
                sftp.get(asset_filename, local_filename)
                sftp.close()
            else:
                logger.info("File already downloaded {}".format(local_filename))
                return True
        except Exception as e:
            logger.exception(
                "Download file from local store failed with error: {}".format(str(e))
            )
            return False
        return True

    def put_file(self, filename):
        """
        Upload a file to the server.

        Args:
            filename (str): The name of the file to upload.

        Returns:
            bool: True if the file was uploaded successfully, False otherwise.
        """
        asset_filename = "{}{}".format(self.location, filename)
        local_filename = os.path.realpath("{}{}".format(self.local_store, filename))
        try:
            local_file = pathlib.Path(local_filename)
            if local_file.is_file():
                sftp = self.ssh.open_sftp()
                sftp.put(local_filename, asset_filename)
                sftp.close()
            else:
                logger.info("Local file does not exist {}".format(local_filename))
                return False
        except Exception as e:
            logger.exception(
                "Download file from local store failed with error: {}".format(str(e))
            )
            return False
        return True

    def del_file(self, filename, *args, **kwargs):
        """
        Delete a file from the server.

        Args:
            filename (str): The name of the file to delete.
            archive (bool): Whether to archive the file (default: False).

        Returns:
            bool: True if the file was deleted successfully, False otherwise.
        """
        asset_filename = "{}{}".format(self.location, filename)
        if os.path.exists(asset_filename):
            try:
                sftp = self.ssh.open_sftp()
                sftp.remove(asset_filename)
                sftp.close()
                return True
            except Exception as e:
                logger.exception(
                    "Delete file from local store failed with error: {}".format(str(e))
                )
        return False

    def __del__(self):
        if self.ssh:
            self.ssh.close()



from assetsstore.assets import FileAssets
import os
import sys
import boto3
import logging
from pathlib import Path
import threading
from botocore.client import Config

logger = logging.getLogger(__name__)


class ProgressPercentage(object):
    def __init__(self, filename, client=None, bucket=None):
        self._filename = filename
        if client:
            self._size = float(
                client.head_object(Bucket=bucket, Key=filename)
                .get("ResponseMetadata", {})
                .get("HTTPHeaders", {})
                .get("content-length", 1)
            )
        else:
            self._size = float(os.path.getsize(filename))

        self._seen_so_far = 0
        self._lock = threading.Lock()

    def __call__(self, bytes_amount):
        # To simplify we'll assume this is hooked up
        # to a single filename.
        with self._lock:
            self._seen_so_far += bytes_amount
            percentage = round((self._seen_so_far / self._size) * 100, 2)
            logger.info(
                "{} is the file name. {} out of {} done. The percentage completed is {} %".format(
                    str(self._filename),
                    str(self._seen_so_far),
                    str(self._size),
                    str(percentage),
                )
            )
            sys.stdout.flush()


class S3Files(FileAssets):
    """
    A class for interacting with files stored in an S3 bucket.

    Attributes:
        aws_access_key_id (str): The AWS access key ID.
        aws_secret_access_key (str): The AWS secret access key.
        s3_bucket_name (str): The name of the S3 bucket.
        region_name (str): The AWS region name.
        connection (boto3.client): The S3 client connection.
        resource (boto3.resource): The S3 resource connection.
    """

    def __init__(
        self,
        access_key=os.getenv("ASSET_ACCESS_KEY"),
        secret_key=os.getenv("ASSET_SECRET_ACCESS_KEY"),
        bucket_name=os.getenv("ASSET_LOCATION"),
        bucket_region=os.getenv("ASSET_REGION"),
    ):
        self.aws_access_key_id = access_key
        self.aws_secret_access_key = secret_key
        self.s3_bucket_name = bucket_name
        self.region_name = bucket_region
        session = None
        if self.aws_access_key_id:
            session = boto3.Session(
                aws_access_key_id=self.aws_access_key_id,
                aws_secret_access_key=self.aws_secret_access_key,
            )
        else:
            session = boto3.Session()
        self.connection = session.client(
            "s3",
            config=Config(region_name=self.region_name, signature_version="s3v4"),
            endpoint_url=f"https://s3.{self.region_name}.amazonaws.com",
        )
        self.resource = session.resource(
            "s3", endpoint_url=f"https://s3.{self.region_name}.amazonaws.com"
        )
        super().__init__()

    def check_if_exists(self, path: str):
        """
        Checks if desired object exists.
        Args:
            path (str): The path in the S3 bucket.

        Returns:
            bool: True if file exists, False otherwise.
        """
        try:
            self.connection.head_object(Bucket=self.s3_bucket_name, Key=path)
            return True
        except Exception as e:
            logger.warning("Cannot access bucket object. Exception {}".format(str(e)))
        return False

    def get_size(self, folder: str):
        """
        Get the total size of files in a folder in the S3 bucket.

        Args:
            folder (str): The folder path in the S3 bucket.

        Returns:
            int: The total size of files in the folder.

        """
        size = 0
        try:
            bucket = self.resource.Bucket(self.s3_bucket_name)
            for key in bucket.objects.filter(Prefix=folder):
                if key.meta.data.get("StorageClass", "") == "STANDARD":
                    size += key.size
        except Exception as e:
            logger.exception(
                "Cannot get size of the S3 bucket folder. Exception: {}".format(str(e))
            )
            return False
        return size

    def get_access(
        self,
        filename: str,
        seconds: int = 0,
        short: bool = False,
        download_filename: str = "",
    ):
        """
        Get temporary access to download a file from the S3 bucket.

        Args:
            filename (str): The name of the file in the S3 bucket.
            seconds (int): The duration of access in seconds (default: 0).
            short (bool): Whether to generate a short URL (default: False).
            download_filename (str): The name of the file to be downloaded (default: "").

        Returns:
            str: The URL for accessing the file.

        """
        response = None
        try:
            if not download_filename:
                download_filename = filename

            if short:
                response = "https://{}.s3.amazonaws.com/{}".format(
                    self.s3_bucket_name, filename
                )
                short_url = self.shorten_url(response)
                if short_url:
                    response = short_url
            else:
                response = self.connection.generate_presigned_url(
                    ClientMethod="get_object",
                    Params={
                        "Bucket": self.s3_bucket_name,
                        "Key": filename,
                        "ResponseContentDisposition": f"attachment;filename={download_filename}",
                    },
                    ExpiresIn=seconds,
                )

        except Exception as e:
            logger.exception(
                "Not able to give access to {} for {} seconds. Exception {}".format(
                    filename, seconds, str(e)
                )
            )
            return False
        return response

    def get_upload_access(self, filename: str, seconds: int = 0):
        """
        Get temporary access to upload a file to the S3 bucket.

        Args:
            filename (str): The name of the file in the S3 bucket.
            seconds (int): The duration of access in seconds (default: 0).

        Returns:
            str: The URL for uploading the file.

        """
        response = None

        # Set the desired multipart threshold value (5GB)
        try:
            response = self.connection.generate_presigned_url(
                ClientMethod="put_object",
                Params={
                    "Bucket": self.s3_bucket_name,
                    "Key": filename,
                },
                ExpiresIn=seconds,
            )

        except Exception as e:
            logger.exception(
                "Not able to give access to {} for {} seconds. Exception {}".format(
                    filename, seconds, str(e)
                )
            )
            return False
        return response

    def get_folder(self, path: str):
        """
        Download a folder from the S3 bucket to the local file system.

        Args:
            path (str): The folder path in the S3 bucket.

        Returns:
            bool: True if the folder is downloaded successfully, False otherwise.

        """
        try:
            local_folder = os.path.realpath("{}{}".format(self.local_store, path))
            logger.info(
                "Getting folder from s3 {}, into local folder {}".format(
                    path, local_folder
                )
            )
            bucket = self.resource.Bucket(self.s3_bucket_name)
            for obj in bucket.objects.filter(Prefix=path):
                try:
                    logger.info("Downloading file {}".format(obj.key))
                    full_filename = os.path.realpath(
                        "{}{}".format(self.local_store, obj.key)
                    )
                    if not os.path.exists(os.path.dirname(full_filename)):
                        os.makedirs(os.path.dirname(full_filename))
                    self.get_file(obj.key)
                except Exception as e:
                    logger.warning(
                        "Error occurred downloading file {}, with error: {}".format(
                            str(e), obj.key
                        )
                    )
        except Exception as e:
            logger.warning(
                "Error occurred while downloading folder from s3 {}".format(str(e))
            )
            return False
        return True

    def del_folder(self, path: str):
        """
        Delete a folder and its contents from the S3 bucket.

        Args:
            path (str): The folder path in the S3 bucket.

        Returns:
            bool: True if the folder is deleted successfully, False otherwise.

        """
        bucket = self.resource.Bucket(self.s3_bucket_name)
        for obj in bucket.objects.filter(Prefix=path):
            try:
                self.del_file(obj.key)
            except Exception as e:
                logger.exception(
                    "Delete file from s3 failed with error: {}".format(str(e))
                )
                return False
        return True

    def get_file(self, filename: str):
        """
        Download a file from the S3 bucket to the local file system.

        Args:
            filename (str): The name of the file in the S3 bucket.

        Returns:
            bool: True if the file is downloaded successfully, False otherwise.

        """
        try:
            full_filename = os.path.realpath("{}{}".format(self.local_store, filename))
            my_file = Path(full_filename)
            if not my_file.is_file():
                folder_path = Path("/".join(full_filename.split("/")[:-1]))
                folder_path.mkdir(parents=True, exist_ok=True)
                progress = ProgressPercentage(
                    filename, self.connection, self.s3_bucket_name
                )
                self.connection.download_file(
                    self.s3_bucket_name, filename, full_filename, Callback=progress
                )
            else:
                logger.info("file already exists at path {}".format(full_filename))
                return True

        except Exception as e:
            logger.exception(
                "Download file from s3 failed with error: {}".format(str(e))
            )
            return False
        return True

    def put_file(self, filename: str):
        """
        Upload a file from the local file system to the S3 bucket.

        Args:
            filename (str): The name of the file in the S3 bucket.

        Returns:
            bool: True if the file is uploaded successfully, False otherwise.

        """
        try:
            full_filename = os.path.realpath("{}{}".format(self.local_store, filename))
            progress = ProgressPercentage(full_filename)
            self.connection.upload_file(
                full_filename, self.s3_bucket_name, filename, Callback=progress
            )
        except Exception as e:
            logger.exception("Upload file to s3 failed with error: {}".format(str(e)))
            return False
        return True

    def del_file(self, filename: str, archive: bool = False):
        """
        Delete a file from the S3 bucket.

        Args:
            filename (str): The name of the file in the S3 bucket.
            archive (bool): Whether to archive the file before deleting (default: False).

        Returns:
            bool: True if the file is deleted successfully, False otherwise.

        """
        try:
            if archive:
                self.connection.copy(
                    {"Bucket": self.s3_bucket_name, "Key": filename},
                    self.s3_bucket_name,
                    filename,
                    ExtraArgs={"StorageClass": "GLACIER", "MetadataDirective": "COPY"},
                )
            else:
                self.connection.delete_object(Bucket=self.s3_bucket_name, Key=filename)

        except Exception as e:
            logger.exception("Delete file from s3 failed with error: {}".format(str(e)))
            return False
        return True



from datetime import timedelta
from assetsstore.assets import FileAssets
import os
import logging
from pathlib import Path
from minio import Minio
from urllib.parse import urlunsplit
from .progress import Progress

logger = logging.getLogger(__name__)


class MinioFiles(FileAssets):
    """
    A class for interacting with Minio file storage.

    This class provides methods for managing files and folders in a Minio bucket.

    Attributes:
        access_key (str): The access key for the Minio server.
        secret_key (str): The secret access key for the Minio server.
        bucket_name (str): The name of the bucket in the Minio server.
        host (str): The host URL of the Minio server.
        tls_enabled (bool): Whether TLS encryption is enabled for the Minio server.
        client (Minio): The Minio client object for interacting with the server.

    """

    def __init__(
        self,
        access_key=os.getenv("ASSET_ACCESS_KEY"),
        secret_key=os.getenv("ASSET_SECRET_ACCESS_KEY"),
        bucket_name=os.getenv("ASSET_LOCATION"),
        bucket_region=os.getenv("ASSET_REGION"),
        host=os.getenv("ASSET_PUBLIC_URL", "localhost:9000"),
        tls_enabled=os.getenv("ASSET_TLS_ENABLED", False),
    ):
        self.access_key = access_key
        self.secret_key = secret_key
        self.bucket_name = bucket_name
        self.host = host
        self.tls_enabled = tls_enabled
        self.client = Minio(
            self.host,
            self.access_key,
            self.secret_key,
            secure=self.tls_enabled,
            region=bucket_region,
        )
        super().__init__()

    def check_if_exists(self, path: str):
        """
        Checks if desired object exists.
        Args:
            path (str): The path in the Minio bucket.

        Returns:
            bool: True if file exists, False otherwise.
        """
        response = None
        try:
            response = self.client.get_object(self.bucket_name, path)
            success = True
        except Exception as e:
            success = False
            logger.warning("Cannot access bucket object. Exception {}".format(str(e)))
        if response:
            response.close()
            response.release_conn()
        return success

    def get_size(self, folder: str):
        """
        Get the total size of a folder in the Minio bucket.

        Args:
            folder (str): The folder path in the Minio bucket.

        Returns:
            int: The total size of the folder in bytes.

        """
        size = 0
        try:
            for obj in self.client.list_objects(
                self.bucket_name, prefix=folder, recursive=True
            ):
                size += obj.size
        except Exception as e:
            logger.exception(
                "Cannot get size of the S3 bucket folder. Exception: {}".format(str(e))
            )
        return size

    def get_access(
        self,
        filename: str,
        seconds: int = 0,
        short=False,
        download_filename: str = "",
    ):
        """
        Get the access URL for a file in the Minio bucket.

        Args:
            filename (str): The name of the file in the Minio bucket.
            seconds (int, optional): The number of seconds the access URL should be valid for. Defaults to 0.
            short (bool, optional): Whether to generate a short URL using a URL shortening service. Defaults to False.

        Returns:
            str: The access URL for the file.

        """
        response = None
        try:
            if not download_filename:
                download_filename = filename
            if short:
                base_url = self.client._base_url._url
                base_url = urlunsplit(base_url)
                response = f"{base_url}/{self.bucket_name}/{filename}"
                short_url = self.shorten_url(response)
                if short_url:
                    response = short_url
            else:
                response = self.client.presigned_get_object(
                    self.bucket_name,
                    filename,
                    response_headers={
                        "response-content-disposition": f"attachment;filename={download_filename}"
                    },
                    expires=timedelta(seconds=seconds if seconds else 604800),
                )
        except Exception as e:
            logger.exception(
                "Not able to give access to {} for {} seconds. Exception {}".format(
                    filename, seconds, str(e)
                )
            )
        return response

    def get_upload_access(self, filename: str, seconds: int = 0):
        """
        Get the access URL for uploading a file to the Minio bucket.

        Args:
            filename (str): The name of the file to be uploaded.
            seconds (int, optional): The number of seconds the access URL should be valid for. Defaults to 0.

        Returns:
            str: The access URL for uploading the file.

        """
        response = None
        try:
            response = self.client.presigned_put_object(
                self.bucket_name,
                filename,
                expires=timedelta(seconds=seconds if seconds else 604800),
            )
        except Exception as e:
            logger.exception(
                "Not able to give access to {} for {} seconds. Exception {}".format(
                    filename, seconds, str(e)
                )
            )
            return False
        return response

    def get_folder(self, path: str):
        """
        Download a folder from the Minio bucket.

        Args:
            path (str): The folder path in the Minio bucket.

        Returns:
            bool: True if the folder is downloaded successfully, False otherwise.

        """
        try:
            local_folder = os.path.realpath("{}{}".format(self.local_store, path))
            logger.info(
                "Getting folder from minio s3 {}, into local folder {}".format(
                    path, local_folder
                )
            )
            for obj in self.client.list_objects(
                self.bucket_name, prefix=path, recursive=True
            ):
                try:
                    logger.info("Downloading file {}".format(obj._object_name))
                    full_filename = os.path.realpath(
                        "{}{}".format(self.local_store, obj._object_name)
                    )
                    if not os.path.exists(os.path.dirname(full_filename)):
                        os.makedirs(os.path.dirname(full_filename))
                    self.get_file(obj)
                except Exception as e:
                    logger.warning(
                        "Error occured downloading file {}, with error: {}".format(
                            str(e), obj._object_name
                        )
                    )
        except Exception as e:
            logger.warning(
                "Error occured while downloading folder from s3 minio {}".format(str(e))
            )
            return False
        return True

    def del_folder(self, path: str):
        """
        Delete a folder from the Minio bucket.

        Args:
            path (str): The folder path in the Minio bucket.

        Returns:
            bool: True if the folder is deleted successfully, False otherwise.

        """
        objects = self.client.list_objects(
            self.bucket_name, prefix=path, recursive=True
        )
        try:
            objects = [x._object_name for x in objects]
            for obj in objects:
                self.del_file(obj)
        except Exception as e:
            logger.exception("Delete file from s3 failed with error: {}".format(str(e)))
            return False
        return True

    def get_file(self, filename: str):
        """
        Download a file from the Minio bucket.

        Args:
            filename (str): The name of the file in the Minio bucket.

        Returns:
            bool: True if the file is downloaded successfully, False otherwise.
        """
        try:
            full_filename = os.path.realpath("{}{}".format(self.local_store, filename))
            my_file = Path(full_filename)
            if not my_file.is_file():
                folder_path = Path("/".join(full_filename.split("/")[:-1]))
                folder_path.mkdir(parents=True, exist_ok=True)
                self.client.fget_object(
                    self.bucket_name, filename, full_filename, progress=Progress()
                )
            else:
                logger.info("file already exists at path {}".format(full_filename))
        except Exception as e:
            logger.exception(
                "Error occurred while downloading file {}. Exception: {}".format(
                    filename, str(e)
                )
            )
            return False
        return True

    def put_file(self, filename: str):
        """
        Upload a file to the Minio bucket.

        Args:
            filename (str): The name of the file in the Minio bucket.

        Returns:
            bool: True if the file is uploaded successfully, False otherwise.
        """
        try:
            full_filename = os.path.realpath("{}{}".format(self.local_store, filename))
            self.client.fput_object(
                self.bucket_name, filename, full_filename, progress=Progress()
            )
            return True
        except Exception as e:
            logger.exception(
                "Upload file to minio s3 failed with error: {}".format(str(e))
            )
            return False

    def del_file(self, filename: str):
        """
        Delete a file from the Minio bucket.

        Args:
            filename (str): The name of the file in the Minio bucket.

        Returns:
            bool: True if the file is deleted successfully, False otherwise.
        """
        try:
            self.client.remove_object(self.bucket_name, filename)
        except Exception as e:
            logger.exception("Delete file from s3 failed with error: {}".format(str(e)))
            return False
        return True



from unittest import TestCase
from assetsstore.assets import FileAssets
import glob
import os
import logging
import json
import requests


class AsssetsLocalTest(TestCase):
    def setUp(self):
        os.environ["ASSET_STORE"] = "MinioFiles"
        os.environ["LOCAL_STORE"] = "assetsstore/tests/fixtures/"
        self.maxDiff = None
        handler = FileAssets.get_asset(
            access_key="minio", secret_key="minio123", bucket_name="test"
        )
        handler.client.make_bucket("test")
        logging.basicConfig(level=logging.INFO)

    def test_upload_and_download_from_minio(self):
        handler = FileAssets.get_asset(
            access_key="minio", secret_key="minio123", bucket_name="test"
        )
        self.assertEqual(True, handler.put_file("test.txt"))

        # change path to avoid exists on 1. step
        os.environ["LOCAL_STORE"] = "assetsstore/tests/results/"

        # download file
        handler = FileAssets.get_asset(
            access_key="minio", secret_key="minio123", bucket_name="test"
        )
        self.assertEqual(True, handler.get_file("test.txt"))

        # get again to check if it exists
        self.assertEqual(True, handler.get_file("test.txt"))

        # delete remote file
        self.assertEqual(True, handler.del_file("test.txt"))

        # delete local copy
        self.assertEqual(True, handler.del_local_file("test.txt"))

    def test_get_folder_from_minio(self):
        handler = FileAssets.get_asset(
            access_key="minio", secret_key="minio123", bucket_name="test"
        )
        self.assertEqual(True, handler.put_file("test_folder/test2.txt"))
        self.assertEqual(True, handler.get_folder("test_folder"))
        self.assertEqual(True, handler.del_file("test_folder/test2.txt"))

    def test_get_upload_access(self):
        handler = FileAssets.get_asset(
            access_key="minio", secret_key="minio123", bucket_name="test"
        )
        self.assertIsNotNone(handler.get_upload_access("test.txt"))

    def test_get_download_access_private_object(self):
        handler = FileAssets.get_asset(
            access_key="minio", secret_key="minio123", bucket_name="test"
        )
        self.assertEqual(True, handler.put_file("test.txt"))
        policy = {
            "Version": "2012-10-17",
            "Statement": [
                {
                    "Effect": "Deny",
                    "Principal": {"AWS": "*"},
                    "Action": "s3:GetObject",
                    "Resource": "arn:aws:s3:::test/test.txt",
                },
                {
                    "Effect": "Deny",
                    "Principal": {"AWS": "*"},
                    "Action": "s3:GetObject",
                    "Resource": "arn:aws:s3:::test/test3.txt",
                },
                {
                    "Effect": "Allow",
                    "Principal": {"AWS": "*"},
                    "Action": "s3:GetObject",
                    "Resource": "arn:aws:s3:::test/test2.txt",
                },
            ],
        }
        self.assertEqual(
            403, requests.get(f"http://{handler.host}/test/test.txt").status_code
        )
        handler.client.set_bucket_policy("test", json.dumps(policy))
        url = handler.get_access("test.txt", short=False, download_filename="novi.txt")
        assert "novi.txt" in url
        self.assertEqual(200, requests.get(url).status_code)
        self.assertEqual(True, handler.del_file("test.txt"))

    def test_get_access_for_public(self):
        handler = FileAssets.get_asset(
            access_key="minio", secret_key="minio123", bucket_name="test"
        )
        self.assertEqual(True, handler.put_file("test_folder/test2.txt"))
        test = handler.get_access("test_folder", short=True)
        self.assertEqual(True, handler.del_file("test_folder/test2.txt"))
        self.assertEqual(test, "http://localhost:9000/test/test_folder")

    def test_get_folder_size(self):
        handler = FileAssets.get_asset(
            access_key="minio", secret_key="minio123", bucket_name="test"
        )
        self.assertEqual(True, handler.put_file("test_folder/test2.txt"))
        self.assertEqual(19, handler.get_size("test_folder"))
        self.assertEqual(True, handler.del_file("test_folder/test2.txt"))

    def test_check_if_file_exists(self):
        handler = FileAssets.get_asset(
            access_key="minio", secret_key="minio123", bucket_name="test"
        )
        self.assertEqual(True, handler.put_file("test_folder/test2.txt"))
        self.assertTrue(handler.check_if_exists("test_folder/test2.txt"))
        self.assertFalse(handler.check_if_exists("test_folder/test33.txt"))
        self.assertEqual(True, handler.del_file("test_folder/test2.txt"))

    def tearDown(self):
        handler = FileAssets.get_asset(
            access_key="minio", secret_key="minio123", bucket_name="test"
        )
        handler.client.remove_bucket("test")
        for file in glob.glob("results/*"):
            if ".gitkeep" not in file:
                os.remove(file)
        for file in glob.glob("results/remote/*"):
            if ".gitkeep" not in file:
                os.remove(file)
